﻿using System;
using System.Diagnostics;
using System.Dynamic;
using System.IO.Pipes;
using System.Reflection;
using System.Runtime.Serialization.Formatters.Binary;
using System.Text;
using System.Linq;

namespace MultiConsole
{
    internal class ConsoleProxyServer : DynamicObject
    {
        private static readonly string CurrentAssemblyName;
        private static readonly string CurrentAsselblyLocation;

        private Process _process;

        static ConsoleProxyServer()
        {
            var currentAssembly = Assembly.GetExecutingAssembly();
            CurrentAssemblyName = currentAssembly.FullName;
            CurrentAsselblyLocation = currentAssembly.Location;
        }

        private readonly NamedPipeServerStream _pipe;

        public ConsoleProxyServer()
        {
            var pipeName = CurrentAssemblyName + ", " + Guid.NewGuid().ToString();
            var encpName = Convert.ToBase64String(Encoding.UTF8.GetBytes(pipeName));

            _pipe = new NamedPipeServerStream(encpName);
            _process = Process.Start(CurrentAsselblyLocation, encpName);

            _pipe.WaitForConnection();
        }

        internal void Shutdown()
        {
            new BinaryFormatter().Serialize(_pipe, new ConsoleProxyCommand
            {
                ControlCmd = ControlCmd.Shutdown
            });

            var response = (ConsoleProxyResponse)new BinaryFormatter().Deserialize(_pipe);
            if (response.Success == false)
                throw new MultiConsoleException(response.FailMessage);

            _pipe.Close();
        }

        public override bool TryInvokeMember(InvokeMemberBinder binder, object[] args, out object result)
        {
            if (binder.Name.StartsWith("Write") && args.Length == 2)
            {
                if (args.Length == 2 && args[1] is Array)
                {
                    new BinaryFormatter().Serialize(_pipe, new ConsoleProxyCommand
                    {
                        ControlCmd = ControlCmd.Invoke,
                        Name = binder.Name,
                        Param = new object[] { string.Format((string)args[0], (object[])args[1]) },
                    });
                }
                else
                {
                    new BinaryFormatter().Serialize(_pipe, new ConsoleProxyCommand
                    {
                        ControlCmd = ControlCmd.Invoke,
                        Name = binder.Name,
                        Param = new object[] { string.Format((string)args[0], args.Skip(1).ToArray()) },
                    });
                }
            }
            else
            {
                new BinaryFormatter().Serialize(_pipe, new ConsoleProxyCommand
                {
                    ControlCmd = ControlCmd.Invoke,
                    Name = binder.Name,
                    Param = args,
                });
            }

            var response = (ConsoleProxyResponse)new BinaryFormatter().Deserialize(_pipe);
            if (response.Success == false)
                throw new MultiConsoleException(response.FailMessage);

            result = response.Return;
            return true;
        }

        public override bool TryGetMember(GetMemberBinder binder, out object result)
        {
            new BinaryFormatter().Serialize(_pipe, new ConsoleProxyCommand
            {
                ControlCmd = ControlCmd.Get,
                Name = binder.Name,
            });

            var response = (ConsoleProxyResponse)new BinaryFormatter().Deserialize(_pipe);
            if (response.Success == false)
                throw new MultiConsoleException(response.FailMessage);

            result = response.Return;
            return true;
        }

        public override bool TrySetMember(SetMemberBinder binder, object value)
        {
            new BinaryFormatter().Serialize(_pipe, new ConsoleProxyCommand
            {
                ControlCmd = ControlCmd.Set,
                Name = binder.Name,
                Param = new[] { value },
            });

            var response = (ConsoleProxyResponse)new BinaryFormatter().Deserialize(_pipe);
            if (response.Success == false)
                throw new MultiConsoleException(response.FailMessage);

            return true;
        }
    }

    internal class MultiConsoleException : Exception
    {
        public MultiConsoleException(string message)
            : base(message)
        {
        }
    }
}