﻿using JetBrains.Annotations;
using System;
using System.Text;

namespace MultiConsole
{
    public interface IConsole
    {
        /// <summary>
        /// 通过控制台扬声器播放提示音。
        /// </summary>
        /// <exception cref="T:System.Security.HostProtectionException">在不允许访问用户界面的服务器（如 SQL Server）上执行此方法。</exception>
        void Beep();

        /// <summary>
        /// 通过控制台扬声器播放具有指定频率和持续时间的提示音。
        /// </summary>
        /// <param name="frequency">提示音的频率，介于 37 到 32767 赫兹之间。</param>
        /// <param name="duration">提示音的持续时间，以毫秒为单位。</param>
        /// <exception cref="T:System.ArgumentOutOfRangeException">
        /// <paramref name="frequency"/> 小于 37 或大于 32767 赫兹。 - 或 - <paramref name="duration"/> 小于或等于零。</exception>
        /// <exception cref="T:System.Security.HostProtectionException">在不允许访问控制台的服务器（如 SQL Server）上执行此方法。</exception>
        void Beep(int frequency, int duration);

        /// <summary>
        /// 清除控制台缓冲区和相应的控制台窗口的显示信息。
        /// </summary>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        void Clear();

        /// <summary>
        /// 将控制台的前景色和背景色设置为默认值。
        /// </summary>
        /// <exception cref="T:System.Security.SecurityException">该用户没有执行此操作的权限。</exception>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        /// <PermissionSet>
        /// <IPermission class="System.Security.Permissions.UIPermission, mscorlib, Version=2.0.3600.0, Culture=neutral, KeyToken=b77a5c561934e089" version="1" Window="SafeTopLevelWindows"/>
        /// </PermissionSet>
        void ResetColor();

        /// <summary>
        /// 将屏幕缓冲区的指定源区域复制到指定的目标区域。
        /// </summary>
        /// <param name="sourceLeft">源区域最左边的列。</param>
        /// <param name="sourceTop">源区域最顶部的行。</param>
        /// <param name="sourceWidth">源区域中列的数目。</param>
        /// <param name="sourceHeight">源区域中的行的数目。</param>
        /// <param name="targetLeft">目标区域最左边的列。</param>
        /// <param name="targetTop">目标区域最顶部的行。</param>
        /// <exception cref="T:System.ArgumentOutOfRangeException">一个或多个参数小于零。 - 或 - <paramref name="sourceLeft"/> 或 <paramref name="targetLeft"/> 大于等于 <see cref="P:System.Console.BufferWidth"/>。 - 或 - <paramref name="sourceTop"/> 或 <paramref name="targetTop"/> 大于等于 <see cref="P:System.Console.BufferHeight"/>。 - 或 - <paramref name="sourceTop"/> + <paramref name="sourceHeight"/> 大于等于 <see cref="P:System.Console.BufferHeight"/>。 - 或 - <paramref name="sourceLeft"/> + <paramref name="sourceWidth"/> 大于等于 <see cref="P:System.Console.BufferWidth"/>。</exception>
        /// <exception cref="T:System.Security.SecurityException">该用户没有执行此操作的权限。</exception>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        /// <PermissionSet>
        /// <IPermission class="System.Security.Permissions.UIPermission, mscorlib, Version=2.0.3600.0, Culture=neutral, KeyToken=b77a5c561934e089" version="1" Window="SafeTopLevelWindows"/>
        /// </PermissionSet>
        void MoveBufferArea(int sourceLeft, int sourceTop, int sourceWidth, int sourceHeight, int targetLeft, int targetTop);

        /// <summary>
        /// 将屏幕缓冲区的指定源区域复制到指定的目标区域。
        /// </summary>
        /// <param name="sourceLeft">源区域最左边的列。</param>
        /// <param name="sourceTop">源区域最顶部的行。</param>
        /// <param name="sourceWidth">源区域中列的数目。</param>
        /// <param name="sourceHeight">源区域中的行的数目。</param>
        /// <param name="targetLeft">目标区域最左边的列。</param>
        /// <param name="targetTop">目标区域最顶部的行。</param>
        /// <param name="sourceChar">用于填充源区域的字符。</param>
        /// <param name="sourceForeColor">用于填充源区域的前景色。</param>
        /// <param name="sourceBackColor">用于填充源区域的背景色。</param>
        /// <exception cref="T:System.ArgumentOutOfRangeException">一个或多个参数小于零。 - 或 - <paramref name="sourceLeft"/> 或 <paramref name="targetLeft"/> 大于等于 <see cref="P:System.Console.BufferWidth"/>。 - 或 - <paramref name="sourceTop"/> 或 <paramref name="targetTop"/> 大于等于 <see cref="P:System.Console.BufferHeight"/>。 - 或 - <paramref name="sourceTop"/> + <paramref name="sourceHeight"/> 大于等于 <see cref="P:System.Console.BufferHeight"/>。 - 或 - <paramref name="sourceLeft"/> + <paramref name="sourceWidth"/> 大于等于 <see cref="P:System.Console.BufferWidth"/>。</exception>
        /// <exception cref="T:System.ArgumentException">一个颜色参数不是或两个颜色参数都不是 <see cref="T:System.ConsoleColor"/> 枚举的成员。</exception>
        /// <exception cref="T:System.Security.SecurityException">该用户没有执行此操作的权限。</exception>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        /// <PermissionSet>
        /// <IPermission class="System.Security.Permissions.UIPermission, mscorlib, Version=2.0.3600.0, Culture=neutral, KeyToken=b77a5c561934e089" version="1" Window="SafeTopLevelWindows"/>
        /// </PermissionSet>
        void MoveBufferArea(int sourceLeft, int sourceTop, int sourceWidth, int sourceHeight, int targetLeft, int targetTop, char sourceChar, ConsoleColor sourceForeColor, ConsoleColor sourceBackColor);

        /// <summary>
        /// 将屏幕缓冲区的高度和宽度设置为指定值。
        /// </summary>
        /// <param name="width">缓冲区的宽度，以列为单位。</param>
        /// <param name="height">缓冲区的高度，以行为单位。</param>
        /// <exception cref="T:System.ArgumentOutOfRangeException">
        /// <paramref name="height"/> 或 <paramref name="width"/> 小于或等于零。 - 或 - <paramref name="height"/> 或 <paramref name="width"/> 大于等于 <see cref="F:System.Int16.MaxValue"/>。 - 或 - <paramref name="width"/> 小于 <see cref="P:System.Console.WindowLeft"/> + <see cref="P:System.Console.WindowWidth"/>。 - 或 - <paramref name="height"/> 小于 <see cref="P:System.Console.WindowTop"/> + <see cref="P:System.Console.WindowHeight"/>。</exception>
        /// <exception cref="T:System.Security.SecurityException">该用户没有执行此操作的权限。</exception>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        /// <PermissionSet>
        /// <IPermission class="System.Security.Permissions.UIPermission, mscorlib, Version=2.0.3600.0, Culture=neutral, KeyToken=b77a5c561934e089" version="1" Window="SafeTopLevelWindows"/>
        /// </PermissionSet>
        void SetBufferSize(int width, int height);

        /// <summary>
        /// 将控制台窗口的高度和宽度设置为指定值。
        /// </summary>
        /// <param name="width">控制台窗口的宽度，以列为单位。</param>
        /// <param name="height">控制台窗口的高度，以行为单位。</param>
        /// <exception cref="T:System.ArgumentOutOfRangeException">
        /// <paramref name="width"/> 或 <paramref name="height"/> 小于或等于零。 - 或 - <paramref name="width"/> 加 <see cref="P:System.Console.WindowLeft"/> 或者 <paramref name="height"/> 加 <see cref="P:System.Console.WindowTop"/> 大于等于 <see cref="F:System.Int16.MaxValue"/>。 - 或 - <paramref name="width"/> 或 <paramref name="height"/> 大于当前屏幕分辨率和控制台字体所允许的最大窗口宽度和高度。</exception>
        /// <exception cref="T:System.Security.SecurityException">该用户没有执行此操作的权限。</exception>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        /// <PermissionSet>
        /// <IPermission class="System.Security.Permissions.UIPermission, mscorlib, Version=2.0.3600.0, Culture=neutral, KeyToken=b77a5c561934e089" version="1" Window="SafeTopLevelWindows"/>
        /// </PermissionSet>
        void SetWindowSize(int width, int height);

        /// <summary>
        /// 设置控制台窗口相对于屏幕缓冲区的位置。
        /// </summary>
        /// <param name="left">控制台窗口左上角的列位置。</param>
        /// <param name="top">控制台窗口左上角的行位置。</param>
        /// <exception cref="T:System.ArgumentOutOfRangeException">
        /// <paramref name="left"/> 或 <paramref name="top"/> 小于零。 - 或 - <paramref name="left"/> + <see cref="P:System.Console.WindowWidth"/> 大于 <see cref="P:System.Console.BufferWidth"/>。 - 或 - <paramref name="top"/> + <see cref="P:System.Console.WindowHeight"/> 大于 <see cref="P:System.Console.BufferHeight"/>。</exception>
        /// <exception cref="T:System.Security.SecurityException">该用户没有执行此操作的权限。</exception>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        /// <PermissionSet>
        /// <IPermission class="System.Security.Permissions.UIPermission, mscorlib, Version=2.0.3600.0, Culture=neutral, KeyToken=b77a5c561934e089" version="1" Window="SafeTopLevelWindows"/>
        /// </PermissionSet>
        void SetWindowPosition(int left, int top);

        /// <summary>
        /// 设置光标位置。
        /// </summary>
        /// <param name="left">光标的列位置。 将从 0 开始从左到右对列进行编号。</param>
        /// <param name="top">光标的行位置。 从上到下，从 0 开始为行编号。</param>
        /// <exception cref="T:System.ArgumentOutOfRangeException">
        /// <paramref name="left"/> 或 <paramref name="top"/> 小于零。 - 或 - <paramref name="left"/> 大于等于 <see cref="P:System.Console.BufferWidth"/>。 - 或 - <paramref name="top"/> 大于等于 <see cref="P:System.Console.BufferHeight"/>。</exception>
        /// <exception cref="T:System.Security.SecurityException">该用户没有执行此操作的权限。</exception>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        /// <PermissionSet>
        /// <IPermission class="System.Security.Permissions.UIPermission, mscorlib, Version=2.0.3600.0, Culture=neutral, KeyToken=b77a5c561934e089" version="1" Window="SafeTopLevelWindows"/>
        /// </PermissionSet>
        void SetCursorPosition(int left, int top);

        /// <summary>
        /// 获取用户按下的下一个字符或功能键。 按下的键显示在控制台窗口中。
        /// </summary>
        ///
        /// <returns>
        /// 一个 <see cref="T:System.ConsoleKeyInfo"/> 对象，描述 <see cref="T:System.ConsoleKey"/> 常数和对应于按下的控制台键的 Unicode 字符（如果存在这样的字符）。 <see cref="T:System.ConsoleKeyInfo"/> 对象还在 <see cref="T:System.ConsoleModifiers"/> 的按按位组合中描述是否在按下控制台键的同时按下了一个或多个 Shift、Alt 和 Ctrl 修改键。
        /// </returns>
        /// <exception cref="T:System.InvalidOperationException">
        /// <see cref="P:System.Console.In"/> 属性是从非控制台的某种流进行重定向的。</exception>
        ConsoleKeyInfo ReadKey();

        /// <summary>
        /// 获取用户按下的下一个字符或功能键。 按下的键可以选择显示在控制台窗口中。
        /// </summary>
        ///
        /// <returns>
        /// 一个 <see cref="T:System.ConsoleKeyInfo"/> 对象，描述 <see cref="T:System.ConsoleKey"/> 常数和对应于按下的控制台键的 Unicode 字符（如果存在这样的字符）。 <see cref="T:System.ConsoleKeyInfo"/> 对象还在 <see cref="T:System.ConsoleModifiers"/> 的按按位组合中描述是否在按下控制台键的同时按下了一个或多个 Shift、Alt 和 Ctrl 修改键。
        /// </returns>
        /// <param name="intercept">确定是否在控制台窗口中显示按下的键。 如果为 true，则不显示按下的键；否则为 false。</param>
        /// <exception cref="T:System.InvalidOperationException">
        /// <see cref="P:System.Console.In"/> 属性是从非控制台的某种流进行重定向的。</exception>
        ConsoleKeyInfo ReadKey(bool intercept);

        /// <summary>
        /// 从标准输入流读取下一个字符。
        /// </summary>
        ///
        /// <returns>
        /// 输入流中的下一个字符；如果当前没有更多的字符可供读取，则为负一 (-1)。
        /// </returns>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        int Read();

        /// <summary>
        /// 从标准输入流读取下一行字符。
        /// </summary>
        ///
        /// <returns>
        /// 输入流中的下一行字符；如果没有更多的可用行，则为 null。
        /// </returns>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        /// <exception cref="T:System.OutOfMemoryException">内存不足，无法为返回的字符串分配缓冲区。</exception>
        /// <exception cref="T:System.ArgumentOutOfRangeException">下一行字符中的字符数大于 <see cref="F:System.Int32.MaxValue"/>。</exception>
        string ReadLine();

        /// <summary>
        /// 将当前行终止符写入标准输出流。
        /// </summary>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        void WriteLine();

        /// <summary>
        /// 将指定布尔值的文本表示形式（后跟当前行终止符）写入标准输出流。
        /// </summary>
        /// <param name="value">要写入的值。</param>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        void WriteLine(bool value);

        /// <summary>
        /// 将指定的 Unicode 字符值（后跟当前行终止符）写入标准输出流。
        /// </summary>
        /// <param name="value">要写入的值。</param>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        void WriteLine(char value);

        /// <summary>
        /// 将指定的 Unicode 字符数组（后跟当前行终止符）写入标准输出流。
        /// </summary>
        /// <param name="buffer">Unicode 字符数组。</param>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        void WriteLine(char[] buffer);

        /// <summary>
        /// 将指定的 Unicode 字符子数组（后跟当前行终止符）写入标准输出流。
        /// </summary>
        /// <param name="buffer">Unicode 字符的数组。</param>
        /// <param name="index">
        /// <paramref name="buffer"/> 中的起始位置。</param>
        /// <param name="count">要写入的字符数。</param>
        /// <exception cref="T:System.ArgumentNullException">
        /// <paramref name="buffer"/> 为 null。</exception>
        /// <exception cref="T:System.ArgumentOutOfRangeException">
        /// <paramref name="index"/> 或 <paramref name="count"/> 小于零。</exception>
        /// <exception cref="T:System.ArgumentException">
        /// <paramref name="index"/> 和 <paramref name="count"/> 共同指定一个不在 <paramref name="buffer"/> 中的位置。</exception>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        void WriteLine(char[] buffer, int index, int count);

        /// <summary>
        /// 将指定的 <see cref="T:System.Decimal"/> 值的文本表示形式（后跟当前行终止符）写入标准输出流。
        /// </summary>
        /// <param name="value">要写入的值。</param>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        void WriteLine(decimal value);

        /// <summary>
        /// 将指定的双精度浮点值的文本表示形式（后跟当前行终止符）写入标准输出流。
        /// </summary>
        /// <param name="value">要写入的值。</param>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        void WriteLine(double value);

        /// <summary>
        /// 将指定的单精度浮点值的文本表示形式（后跟当前行终止符）写入标准输出流。
        /// </summary>
        /// <param name="value">要写入的值。</param>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        void WriteLine(float value);

        /// <summary>
        /// 将指定的 32 位有符号整数值的文本表示（后跟当前行的结束符）写入标准输出流。
        /// </summary>
        /// <param name="value">要写入的值。</param>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        void WriteLine(int value);

        /// <summary>
        /// 将指定的 32 位无符号的整数值的文本表示（后跟当前行的结束符）写入标准输出流。
        /// </summary>
        /// <param name="value">要写入的值。</param>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        void WriteLine(uint value);

        /// <summary>
        /// 将指定的 64 位有符号整数值的文本表示（后跟当前行的结束符）写入标准输出流。
        /// </summary>
        /// <param name="value">要写入的值。</param>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        void WriteLine(long value);

        /// <summary>
        /// 将指定的 64 位无符号的整数值的文本表示（后跟当前行的结束符）写入标准输出流。
        /// </summary>
        /// <param name="value">要写入的值。</param>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        void WriteLine(ulong value);

        /// <summary>
        /// 将指定对象的文本表示形式（后跟当前行终止符）写入标准输出流。
        /// </summary>
        /// <param name="value">要写入的值。</param>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        void WriteLine(object value);

        /// <summary>
        /// 将指定的字符串值（后跟当前行终止符）写入标准输出流。
        /// </summary>
        /// <param name="value">要写入的值。</param>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        void WriteLine(string value);

        /// <summary>
        /// 使用指定的格式信息，将指定对象（后跟当前行终止符）的文本表示形式写入标准输出流。
        /// </summary>
        /// <param name="format">复合格式字符串（请参见“备注”）。</param>
        /// <param name="arg0">要使用 <paramref name="format"/> 写入的对象。</param>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        /// <exception cref="T:System.ArgumentNullException">
        /// <paramref name="format"/> 为 null。</exception>
        /// <exception cref="T:System.FormatException">
        /// <paramref name="format"/> 中的格式规范无效。</exception>
        [StringFormatMethod("format")]
        void WriteLine(string format, object arg0);

        /// <summary>
        /// 使用指定的格式信息，将指定对象的文本表示形式（后跟当前行终止符）写入标准输出流。
        /// </summary>
        /// <param name="format">复合格式字符串（请参见“备注”）。</param>
        /// <param name="arg0">要使用 <paramref name="format"/> 写入的第一个对象。</param>
        /// <param name="arg1">要使用 <paramref name="format"/> 写入的第二个对象。</param>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        /// <exception cref="T:System.ArgumentNullException">
        /// <paramref name="format"/> 为 null。</exception>
        /// <exception cref="T:System.FormatException">
        /// <paramref name="format"/> 中的格式规范无效。</exception>
        [StringFormatMethod("format")]
        void WriteLine(string format, object arg0, object arg1);

        /// <summary>
        /// 使用指定的格式信息，将指定对象的文本表示形式（后跟当前行终止符）写入标准输出流。
        /// </summary>
        /// <param name="format">复合格式字符串（请参见“备注”）。</param>
        /// <param name="arg0">要使用 <paramref name="format"/> 写入的第一个对象。</param>
        /// <param name="arg1">要使用 <paramref name="format"/> 写入的第二个对象。</param>
        /// <param name="arg2">要使用 <paramref name="format"/> 写入的第三个对象。</param>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        /// <exception cref="T:System.ArgumentNullException">
        /// <paramref name="format"/> 为 null。</exception>
        /// <exception cref="T:System.FormatException">
        /// <paramref name="format"/> 中的格式规范无效。</exception>
        [StringFormatMethod("format")]
        void WriteLine(string format, object arg0, object arg1, object arg2);

        /// <summary>
        /// 使用指定的格式信息，将指定的对象数组（后跟当前行终止符）的文本表示形式写入标准输出流。
        /// </summary>
        /// <param name="format">复合格式字符串（请参见“备注”）。</param>
        /// <param name="arg">要使用 <paramref name="format"/> 写入的对象的数组。</param>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        /// <exception cref="T:System.ArgumentNullException">
        /// <paramref name="format"/> 或 <paramref name="arg"/> 为 null。</exception>
        /// <exception cref="T:System.FormatException">
        /// <paramref name="format"/> 中的格式规范无效。</exception>
        [StringFormatMethod("format")]
        void WriteLine(string format, params object[] arg);

        /// <summary>
        /// 使用指定的格式信息将指定对象的文本表示形式写入标准输出流。
        /// </summary>
        /// <param name="format">复合格式字符串（请参见“备注”）。</param>
        /// <param name="arg0">要使用 <paramref name="format"/> 写入的对象。</param>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        /// <exception cref="T:System.ArgumentNullException">
        /// <paramref name="format"/> 为 null。</exception>
        /// <exception cref="T:System.FormatException">
        /// <paramref name="format"/> 中的格式规范无效。</exception>
        [StringFormatMethod("format")]
        void Write(string format, object arg0);

        /// <summary>
        /// 使用指定的格式信息将指定对象的文本表示形式写入标准输出流。
        /// </summary>
        /// <param name="format">复合格式字符串（请参见“备注”）。</param>
        /// <param name="arg0">要使用 <paramref name="format"/> 写入的第一个对象。</param>
        /// <param name="arg1">要使用 <paramref name="format"/> 写入的第二个对象。</param>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        /// <exception cref="T:System.ArgumentNullException">
        /// <paramref name="format"/> 为 null。</exception>
        /// <exception cref="T:System.FormatException">
        /// <paramref name="format"/> 中的格式规范无效。</exception>
        [StringFormatMethod("format")]
        void Write(string format, object arg0, object arg1);

        /// <summary>
        /// 使用指定的格式信息将指定对象的文本表示形式写入标准输出流。
        /// </summary>
        /// <param name="format">复合格式字符串（请参见“备注”）。</param>
        /// <param name="arg0">要使用 <paramref name="format"/> 写入的第一个对象。</param>
        /// <param name="arg1">要使用 <paramref name="format"/> 写入的第二个对象。</param>
        /// <param name="arg2">要使用 <paramref name="format"/> 写入的第三个对象。</param>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        /// <exception cref="T:System.ArgumentNullException">
        /// <paramref name="format"/> 为 null。</exception>
        /// <exception cref="T:System.FormatException">
        /// <paramref name="format"/> 中的格式规范无效。</exception>
        [StringFormatMethod("format")]
        void Write(string format, object arg0, object arg1, object arg2);

        /// <summary>
        /// 使用指定的格式信息将指定的对象数组的文本表示形式写入标准输出流。
        /// </summary>
        /// <param name="format">复合格式字符串（请参见“备注”）。</param>
        /// <param name="arg">要使用 <paramref name="format"/> 写入的对象的数组。</param>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        /// <exception cref="T:System.ArgumentNullException">
        /// <paramref name="format"/> 或 <paramref name="arg"/> 为 null。</exception>
        /// <exception cref="T:System.FormatException">
        /// <paramref name="format"/> 中的格式规范无效。</exception>
        [StringFormatMethod("format")]
        void Write(string format, params object[] arg);

        /// <summary>
        /// 将指定的布尔值的文本表示形式写入标准输出流。
        /// </summary>
        /// <param name="value">要写入的值。</param>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        void Write(bool value);

        /// <summary>
        /// 将指定的 Unicode 字符值写入标准输出流。
        /// </summary>
        /// <param name="value">要写入的值。</param>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        void Write(char value);

        /// <summary>
        /// 将指定的 Unicode 字符数组写入标准输出流。
        /// </summary>
        /// <param name="buffer">Unicode 字符数组。</param>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        void Write(char[] buffer);

        /// <summary>
        /// 将指定的 Unicode 字符子数组写入标准输出流。
        /// </summary>
        /// <param name="buffer">Unicode 字符的数组。</param>
        /// <param name="index">
        /// <paramref name="buffer"/> 中的起始位置。</param>
        /// <param name="count">要写入的字符数。</param>
        /// <exception cref="T:System.ArgumentNullException">
        /// <paramref name="buffer"/> 为 null。</exception>
        /// <exception cref="T:System.ArgumentOutOfRangeException">
        /// <paramref name="index"/> 或 <paramref name="count"/> 小于零。</exception>
        /// <exception cref="T:System.ArgumentException">
        /// <paramref name="index"/> 和 <paramref name="count"/> 共同指定一个不在 <paramref name="buffer"/> 中的位置。</exception>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        void Write(char[] buffer, int index, int count);

        /// <summary>
        /// 将指定的双精度浮点值的文本表示形式写入标准输出流。
        /// </summary>
        /// <param name="value">要写入的值。</param>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        void Write(double value);

        /// <summary>
        /// 将指定的 <see cref="T:System.Decimal"/> 值的文本表示形式写入标准输出流。
        /// </summary>
        /// <param name="value">要写入的值。</param>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        void Write(decimal value);

        /// <summary>
        /// 将指定的单精度浮点值的文本表示形式写入标准输出流。
        /// </summary>
        /// <param name="value">要写入的值。</param>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        void Write(float value);

        /// <summary>
        /// 将指定的 32 位有符号整数值的文本表示写入标准输出流。
        /// </summary>
        /// <param name="value">要写入的值。</param>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        void Write(int value);

        /// <summary>
        /// 将指定的 32 位无符号整数值的文本表示写入标准输出流。
        /// </summary>
        /// <param name="value">要写入的值。</param>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        void Write(uint value);

        /// <summary>
        /// 将指定的 64 位有符号整数值的文本表示写入标准输出流。
        /// </summary>
        /// <param name="value">要写入的值。</param>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        void Write(long value);

        /// <summary>
        /// 将指定的 64 位无符号整数值的文本表示写入标准输出流。
        /// </summary>
        /// <param name="value">要写入的值。</param>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        void Write(ulong value);

        /// <summary>
        /// 将指定对象的文本表示形式写入标准输出流。
        /// </summary>
        /// <param name="value">要写入的值，或者为 null。</param>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        void Write(object value);

        /// <summary>
        /// 将指定的字符串值写入标准输出流。
        /// </summary>
        /// <param name="value">要写入的值。</param>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        void Write(string value);

        /// <summary>
        /// 获取指示输入是否已从标准输入流中重定向的值。
        /// </summary>
        ///
        /// <returns>
        /// 如果重定向输入，则为 true；否则为 false。
        /// </returns>
        bool IsInputRedirected { get; }

        /// <summary>
        /// 获取指示输出是否已从标准输入流中重定向的值。
        /// </summary>
        ///
        /// <returns>
        /// 如果重定向输出，则为 true；否则为 false。
        /// </returns>
        bool IsOutputRedirected { get; }

        /// <summary>
        /// 获取指示错误输出流是否已经从标准错误流被再定位的值。
        /// </summary>
        ///
        /// <returns>
        /// 如果重定向错误，则为 true；否则为 false。
        /// </returns>
        bool IsErrorRedirected { get; }

        /// <summary>
        /// 获取或设置控制台用于读取输入的编码。
        /// </summary>
        ///
        /// <returns>
        /// 用于读取控制台输入的编码。
        /// </returns>
        /// <exception cref="T:System.ArgumentNullException">Set 操作中的属性值为 null。</exception>
        /// <exception cref="T:System.IO.IOException">在执行此操作的过程中发生错误。</exception>
        /// <exception cref="T:System.Security.SecurityException">您的应用程序不具有执行此操作的权限。</exception>
        Encoding InputEncoding { get; set; }

        /// <summary>
        /// 获取或设置控制台用于写入输出的编码。
        /// </summary>
        ///
        /// <returns>
        /// 用于写入控制台输出的编码。
        /// </returns>
        /// <exception cref="T:System.ArgumentNullException">Set 操作中的属性值为 null。</exception>
        /// <exception cref="T:System.IO.IOException">在执行此操作的过程中发生错误。</exception>
        /// <exception cref="T:System.Security.SecurityException">您的应用程序不具有执行此操作的权限。</exception>
        Encoding OutputEncoding { get; set; }

        /// <summary>
        /// 获取或设置控制台的背景色。
        /// </summary>
        ///
        /// <returns>
        /// 一个值，指定控制台的背景色；也就是显示在每个字符后面的颜色。 默认为黑色。
        /// </returns>
        /// <exception cref="T:System.ArgumentException">在 Set 操作中指定的颜色不是 <see cref="T:System.ConsoleColor"/> 的有效成员。</exception>
        /// <exception cref="T:System.Security.SecurityException">该用户没有执行此操作的权限。</exception>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        /// <PermissionSet>
        /// <IPermission class="System.Security.Permissions.UIPermission, mscorlib, Version=2.0.3600.0, Culture=neutral, KeyToken=b77a5c561934e089" version="1" Window="SafeTopLevelWindows"/>
        /// </PermissionSet>
        ConsoleColor BackgroundColor { get; set; }

        /// <summary>
        /// 获取或设置控制台的前景色。
        /// </summary>
        ///
        /// <returns>
        /// 一个 <see cref="T:System.ConsoleColor"/>，指定控制台的前景色；也就是显示的每个字符的颜色。 默认为灰色。
        /// </returns>
        /// <exception cref="T:System.ArgumentException">在 Set 操作中指定的颜色不是 <see cref="T:System.ConsoleColor"/> 的有效成员。</exception>
        /// <exception cref="T:System.Security.SecurityException">该用户没有执行此操作的权限。</exception>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        /// <PermissionSet>
        /// <IPermission class="System.Security.Permissions.UIPermission, mscorlib, Version=2.0.3600.0, Culture=neutral, KeyToken=b77a5c561934e089" version="1" Window="SafeTopLevelWindows"/>
        /// </PermissionSet>
        ConsoleColor ForegroundColor { get; set; }

        /// <summary>
        /// 获取或设置缓冲区的高度。
        /// </summary>
        ///
        /// <returns>
        /// 缓冲区的当前高度，以行为单位。
        /// </returns>
        /// <exception cref="T:System.ArgumentOutOfRangeException">Set 操作中的值小于或等于零。 - 或 - Set 操作中的值大于等于 <see cref="F:System.Int16.MaxValue"/>。 - 或 - Set 操作中的值小于 <see cref="P:System.Console.WindowTop"/> + <see cref="P:System.Console.WindowHeight"/>。</exception>
        /// <exception cref="T:System.Security.SecurityException">该用户没有执行此操作的权限。</exception>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        int BufferHeight { get; set; }

        /// <summary>
        /// 获取或设置缓冲区的宽度。
        /// </summary>
        ///
        /// <returns>
        /// 缓冲区的当前宽度，以列为单位。
        /// </returns>
        /// <exception cref="T:System.ArgumentOutOfRangeException">Set 操作中的值小于或等于零。 - 或 - Set 操作中的值大于等于 <see cref="F:System.Int16.MaxValue"/>。 - 或 - Set 操作中的值小于 <see cref="P:System.Console.WindowLeft"/> + <see cref="P:System.Console.WindowWidth"/>。</exception>
        /// <exception cref="T:System.Security.SecurityException">该用户没有执行此操作的权限。</exception>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        int BufferWidth { get; set; }

        /// <summary>
        /// 获取或设置控制台窗口区域的高度。
        /// </summary>
        ///
        /// <returns>
        /// 控制台窗口的高度，以行为单位。
        /// </returns>
        /// <exception cref="T:System.ArgumentOutOfRangeException">
        /// <see cref="P:System.Console.WindowWidth"/> 属性值或 <see cref="P:System.Console.WindowHeight"/> 属性值小于或等于 0。 - 或 - <see cref="P:System.Console.WindowHeight"/> 属性值加上 <see cref="P:System.Console.WindowTop"/> 属性值大于等于 <see cref="F:System.Int16.MaxValue"/>。 - 或 - <see cref="P:System.Console.WindowWidth"/> 属性值或 <see cref="P:System.Console.WindowHeight"/> 属性值大于当前屏幕分辨率和控制台字体的最大可能窗口宽度或高度。</exception>
        /// <exception cref="T:System.IO.IOException">读取或写入信息时发生错误。</exception>
        int WindowHeight { get; set; }

        /// <summary>
        /// 获取或设置控制台窗口的宽度。
        /// </summary>
        ///
        /// <returns>
        /// 控制台窗口的宽度，以列为单位。
        /// </returns>
        /// <exception cref="T:System.ArgumentOutOfRangeException">
        /// <see cref="P:System.Console.WindowWidth"/> 属性值或 <see cref="P:System.Console.WindowHeight"/> 属性值小于或等于 0。 - 或 - <see cref="P:System.Console.WindowHeight"/> 属性值加上 <see cref="P:System.Console.WindowTop"/> 属性值大于等于 <see cref="F:System.Int16.MaxValue"/>。 - 或 - <see cref="P:System.Console.WindowWidth"/> 属性值或 <see cref="P:System.Console.WindowHeight"/> 属性值大于当前屏幕分辨率和控制台字体的最大可能窗口宽度或高度。</exception>
        /// <exception cref="T:System.IO.IOException">读取或写入信息时发生错误。</exception>
        int WindowWidth { get; set; }

        /// <summary>
        /// 根据当前字体和屏幕分辨率获取控制台窗口可能具有的最大列数。
        /// </summary>
        ///
        /// <returns>
        /// 控制台窗口可能具有的最大宽度，以列为单位。
        /// </returns>
        int LargestWindowWidth { get; }

        /// <summary>
        /// 根据当前字体和屏幕分辨率获取控制台窗口可能具有的最大行数。
        /// </summary>
        ///
        /// <returns>
        /// 控制台窗口可能具有的最大高度，以行为单位。
        /// </returns>
        int LargestWindowHeight { get; }

        /// <summary>
        /// 获取或设置控制台窗口区域的最左边相对于屏幕缓冲区的位置。
        /// </summary>
        ///
        /// <returns>
        /// 控制台窗口的最左边的位置，以列为单位。
        /// </returns>
        /// <exception cref="T:System.ArgumentOutOfRangeException">在设置操作中，所赋的值小于零。 - 或 - 赋值后，<see cref="P:System.Console.WindowLeft"/> 加上 <see cref="P:System.Console.WindowWidth"/> 将超过 <see cref="P:System.Console.BufferWidth"/>。</exception>
        /// <exception cref="T:System.IO.IOException">读取或写入信息时发生错误。</exception>
        int WindowLeft { get; set; }

        /// <summary>
        /// 获取或设置控制台窗口区域的最顶部相对于屏幕缓冲区的位置。
        /// </summary>
        ///
        /// <returns>
        /// 控制台窗口最顶部的位置，以行为单位。
        /// </returns>
        /// <exception cref="T:System.ArgumentOutOfRangeException">在设置操作中，所赋的值小于零。 - 或 - 赋值后，<see cref="P:System.Console.WindowTop"/> 加上 <see cref="P:System.Console.WindowHeight"/> 将超过 <see cref="P:System.Console.BufferHeight"/>。</exception>
        /// <exception cref="T:System.IO.IOException">读取或写入信息时发生错误。</exception>
        int WindowTop { get; set; }

        /// <summary>
        /// 获取或设置光标在缓冲区中的列位置。
        /// </summary>
        ///
        /// <returns>
        /// 光标的当前位置，以列为单位。
        /// </returns>
        /// <exception cref="T:System.ArgumentOutOfRangeException">Set 操作中的值小于零。 - 或 - Set 操作中的值大于等于 <see cref="P:System.Console.BufferWidth"/>。</exception>
        /// <exception cref="T:System.Security.SecurityException">该用户没有执行此操作的权限。</exception>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        int CursorLeft { get; set; }

        /// <summary>
        /// 获取或设置光标在缓冲区中的行位置。
        /// </summary>
        ///
        /// <returns>
        /// 光标的当前位置，以行为单位。
        /// </returns>
        /// <exception cref="T:System.ArgumentOutOfRangeException">Set 操作中的值小于零。 - 或 - Set 操作中的值大于等于 <see cref="P:System.Console.BufferHeight"/>。</exception>
        /// <exception cref="T:System.Security.SecurityException">该用户没有执行此操作的权限。</exception>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        int CursorTop { get; set; }

        /// <summary>
        /// 获取或设置光标在字符单元格中的高度。
        /// </summary>
        ///
        /// <returns>
        /// 光标的大小，以字符单元格高度的百分比表示。 属性值的范围为 1 到 100。
        /// </returns>
        /// <exception cref="T:System.ArgumentOutOfRangeException">在 Set 操作中指定的值小于 1 或大于 100。</exception>
        /// <exception cref="T:System.Security.SecurityException">该用户没有执行此操作的权限。</exception>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        /// <PermissionSet>
        /// <IPermission class="System.Security.Permissions.UIPermission, mscorlib, Version=2.0.3600.0, Culture=neutral, KeyToken=b77a5c561934e089" version="1" Window="SafeTopLevelWindows"/>
        /// </PermissionSet>
        int CursorSize { get; set; }

        /// <summary>
        /// 获取或设置一个值，用以指示光标是否可见。
        /// </summary>
        ///
        /// <returns>
        /// 如果光标可见，则为 true；否则为 false。
        /// </returns>
        /// <exception cref="T:System.Security.SecurityException">该用户没有执行此操作的权限。</exception>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        /// <PermissionSet>
        /// <IPermission class="System.Security.Permissions.UIPermission, mscorlib, Version=2.0.3600.0, Culture=neutral, KeyToken=b77a5c561934e089" version="1" Window="SafeTopLevelWindows"/>
        /// </PermissionSet>
        bool CursorVisible { get; set; }

        /// <summary>
        /// 获取或设置要显示在控制台标题栏中的标题。
        /// </summary>
        ///
        /// <returns>
        /// 要在控制台的标题栏中显示的字符串。 标题字符串的最大长度是 24500 个字符。
        /// </returns>
        /// <exception cref="T:System.InvalidOperationException">在 Get 操作中，检索到的标题的长度超过 24500 个字符。</exception>
        /// <exception cref="T:System.ArgumentOutOfRangeException">在 Set 操作中，指定标题的长度超过 24500 个字符。</exception>
        /// <exception cref="T:System.ArgumentNullException">在 Set 操作中，指定的标题为 null。</exception>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        /// <PermissionSet>
        /// <IPermission class="System.Security.Permissions.UIPermission, mscorlib, Version=2.0.3600.0, Culture=neutral, KeyToken=b77a5c561934e089" version="1" Window="SafeTopLevelWindows"/>
        /// </PermissionSet>
        string Title { get; set; }

        /// <summary>
        /// 获取一个值，该值指示按键操作在输入流中是否可用。
        /// </summary>
        ///
        /// <returns>
        /// 如果按键操作可用，则为 true；否则为 false。
        /// </returns>
        /// <exception cref="T:System.IO.IOException">发生了 I/O 错误。</exception>
        /// <exception cref="T:System.InvalidOperationException">标准输入重定向到文件而不是键盘。</exception>
        bool KeyAvailable { get; }

        /// <summary>
        /// 获取一个值，该值指示 Num Lock 键盘切换键是打开的还是关闭的。
        /// </summary>
        ///
        /// <returns>
        /// 如果 Num Lock 是打开的，则为 true；如果 Num Lock 是关闭的，则为 false。
        /// </returns>
        bool NumberLock { get; }

        /// <summary>
        /// 获取一个值，该值指示 Caps Lock 键盘切换键是打开的还是关闭的。
        /// </summary>
        ///
        /// <returns>
        /// 如果 Caps Lock 是打开的，则为 true；如果 Caps Lock 是关闭的，则为 false。
        /// </returns>
        bool CapsLock { get; }

        /// <summary>
        /// 获取或设置一个值，该值指示是将修改键 <see cref="F:System.ConsoleModifiers.Control"/> 和控制台键 <see cref="F:System.ConsoleKey.C"/> 的组合 (Ctrl+C) 视为普通输入，还是视为由操作系统处理的中断。
        /// </summary>
        ///
        /// <returns>
        /// 如果将 Ctrl+C 视为普通输入，则为 true；否则为 false。
        /// </returns>
        /// <exception cref="T:System.IO.IOException">无法获取或设置控制台输入缓冲区的输入模式。</exception>
        /// <PermissionSet>
        /// <IPermission class="System.Security.Permissions.UIPermission, mscorlib, Version=2.0.3600.0, Culture=neutral, KeyToken=b77a5c561934e089" version="1" Window="SafeTopLevelWindows"/>
        /// </PermissionSet>
        bool TreatControlCAsInput { get; set; }
    }
}